# -----------------------------------------------------------------
# synth.tcl
#
# 1/23/2011 D. W. Hawkins (dwh@ovro.caltech.edu)
#
# Quartus synthesis script.
#
# -----------------------------------------------------------------
# Usage
# -----
#
# 1. From within Quartus, change to the project folder, and type
#
#    source scripts/synth.tcl
#
# 2. Command-line processing. Change to the project folder, 
#    and type either;
# 
#    a) quartus_sh -s
#       tcl> source scripts/synth.tcl
#
#    b)  quartus_sh -t scripts/synth.tcl
#
# -----------------------------------------------------------------
# References
# ----------
#
# [1] Altera. "Quartus II Scripting Reference Manual",
#     v9.1, 2009.
#
# [2] Altera. "Quartus II Settings File Reference Manual", 
#     v7.0, 2010.
#
#
# -----------------------------------------------------------------

puts ""
puts "Synthesizing the BeMicro-SDK 'example1' design"
puts "----------------------------------------------"

package require ::quartus::project
package require ::quartus::flow

# -----------------------------------------------------------------
# Design paths
# -----------------------------------------------------------------

# Design parameters
set board      bemicro_sdk
set design     example1

# Design paths
set device_top  [pwd]
set constraints $device_top/../share/scripts/constraints.tcl
set design_top  $device_top/
set scripts     $design_top/scripts
set src         $design_top/src

# -----------------------------------------------------------------
# Quartus work
# -----------------------------------------------------------------

global quartus
puts " - Quartus $quartus(version)"

# Quartus build directory
set qwork  $device_top/../build/$design

if {![file exists $qwork]} {
    puts " - Creating the Quartus work directory"
    puts "   * $qwork"
    file mkdir $qwork
}

# Create all the generated files in the work directory    
cd $qwork

# -----------------------------------------------------------------
# Design files
# -----------------------------------------------------------------

puts " - Creating the design files list"

# Close any open project
# * since all the BeMicro-SDK projects are named bemicro_sdk, close
#   the current project to clear the files list. This avoids the
#   top-level files from another BeMicro-SDK project being picked
#   up if the previous project was not closed.
#
if {[is_project_open]} {
	puts "   * close the project"
	project_close
}

# Best to name the project your "top" component name.
#
#  * $quartus(project) contains the project name
#  * project_exist bemicro_sdk returns 1 only in the work directory,
#    since that is where the Quartus project file is located
#
if {[project_exists bemicro_sdk]} {
	puts "   * open the existing bemicro_sdk project"
	project_open -revision bemicro_sdk bemicro_sdk
} else {
	puts "   * create a new bemicro_sdk project"
	project_new -revision bemicro_sdk bemicro_sdk
}

# Create a list of VHDL files to build
#
# The components library files
if {[info exists vfiles]} {
	unset vfiles
}
set vfiles ""

# Add the design files
lappend vfiles $src/blink_leds.vhd
lappend vfiles $src/bemicro_sdk.vhd

# Pass the VHDL files list to Quartus
foreach vfile $vfiles {
    set_global_assignment -name VHDL_FILE $vfile
}

# -----------------------------------------------------------------
# Design constraints
# -----------------------------------------------------------------

puts " - Applying constraints"
source $constraints
set_default_constraints

# SDC constraints
set_global_assignment -name SDC_FILE $scripts/bemicro_sdk.sdc

# Set the blink LEDs generics
# Note:
#   Quartus passes 50.0e6 as a string to the generic, which is
#   not correct. Use Tcl to convert from scientific notation to
#   a format that gets passed to the VHDL correctly. The value
#   can be seen in Quartus under:
#
#     Assignments->Settings, Analysis & Synthesis Settings,
#     Default Parameters.
#
set_parameter -name CLK_FREQUENCY [expr {double(50.0e6)}]
set_parameter -name BLINK_PERIOD  1.0

# -----------------------------------------------------------------
# Process the design
# -----------------------------------------------------------------

puts " - Processing the design"

execute_flow -compile

# Use one of the following to save the settings
#project_close
export_assignments

# Return to the top directory    
cd $design_top

puts " - Processing completed"
puts ""

