-- ----------------------------------------------------------------
-- blink_leds.vhd
--
-- 1/23/2011 D. W. Hawkins (dwh@ovro.caltech.edu)
--
-- A component to blink LEDs.
--
-- ----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

-- ----------------------------------------------------------------

entity blink_leds is
	generic (
		-- Clock frequency
		CLK_FREQUENCY : real := 50.0e6;

		-- LED LSB blink period
		BLINK_PERIOD  : real := 0.5;

		-- Number of LEDs
		LED_WIDTH     : integer := 8;

		-- LEDs are active high or low?
		ACTIVE_HIGH   : boolean := true
	);
	port (
		rstN : in  std_logic;
		clk  : in  std_logic;
		led  : out std_logic_vector(LED_WIDTH-1 downto 0)
	);
end entity;

-- ----------------------------------------------------------------

architecture mixed of blink_leds is

	-- ------------------------------------------------------------
	-- Parameters
	-- ------------------------------------------------------------
	--
	-- Counter count value
	constant COUNT_VALUE : real := BLINK_PERIOD*CLK_FREQUENCY;

	-- Counter width required
	constant COUNT_WIDTH : integer :=
		integer(ceil(log2(COUNT_VALUE-1.0))) + LED_WIDTH - 1;

	-- ------------------------------------------------------------
	-- Signals
	-- ------------------------------------------------------------
	--
	-- Counter
	signal count : unsigned(COUNT_WIDTH-1 downto 0);

	-- Most significant bits of the counter
	alias led_count is
		count(COUNT_WIDTH-1 downto COUNT_WIDTH-LED_WIDTH);

begin

	-- ------------------------------------------------------------
	-- Generics
	-- ------------------------------------------------------------
	--
	-- Report the values of the generics.
	--
	assert false
		report
		       lf &
		       " => CLK_FREQUENCY = " & real'image(CLK_FREQUENCY) &
		       lf &
		       " => BLINK_PERIOD = " & real'image(BLINK_PERIOD) &
		       lf &
		       " => COUNT_VALUE = " & real'image(COUNT_VALUE) &
		       lf &
		       " => COUNT_WIDTH = " & integer'image(COUNT_WIDTH) &
		       lf
		severity note;

	-- ------------------------------------------------------------
	-- LED counter
	-- ------------------------------------------------------------
	--
	process(clk, rstN)
	begin
		if (rstN = '0') then
			count <= (others => '0');
		elsif rising_edge(clk) then
			count <= count + 1;
		end if;
	end process;

	-- ------------------------------------------------------------
	-- LED output
	-- ------------------------------------------------------------
	--
	g1: if (ACTIVE_HIGH) generate
		led <= std_logic_vector(led_count);
	end generate;

	g2: if (not ACTIVE_HIGH) generate
		led <= not std_logic_vector(led_count);
	end generate;

end architecture;
