-- ----------------------------------------------------------------
-- bemicro_sdk.vhd
--
-- 1/23/2011 D. W. Hawkins (dwh@ovro.caltech.edu)
--
-- Arrow BeMicro-SDK Cyclone IV FPGA USB stick 'basic' design.
--
-- * LED count (using the 50MHz oscillator)
-- * No use of the Mobile DDR or GPIOs
--
-- This top-level design file can be used as the template for all
-- BeMicro-SDK designs. This design file defines the pin directions
-- for all devices on the board (including the unused devices).
--
-- ----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

-- ----------------------------------------------------------------

entity bemicro_sdk is
	generic (
		-- Clock frequency
		CLK_FREQUENCY : real := 50.0e6;

		-- LED LSB blink period
		BLINK_PERIOD  : real := 0.5;

		-- Counter implementation
		LIB_VENDOR    : string := "ALTERA";  -- GENERIC or ALTERA
		LIB_TYPE      : string := "GENERIC"  -- GENERIC or LPM

	);
	port (
		-- --------------------------------------------------------
		-- Clock
		-- --------------------------------------------------------
		--
		-- 50MHz oscillator
		clkin_50MHz : in    std_logic;

		-- --------------------------------------------------------
		-- User I/O
		-- --------------------------------------------------------
		--
		-- Push buttons
		cpu_rstN    : in    std_logic;
		pb          : in    std_logic;

		-- Switches (the PCB has the switch numbers)
		sw          : in    std_logic_vector(2 downto 1);

		-- LEDs
		led         : out   std_logic_vector(7 downto 0);

		-- --------------------------------------------------------
		-- SPI temperature sensor
		-- --------------------------------------------------------
		--
		spi_sck     : out   std_logic;
		spi_csN     : out   std_logic;
		spi_mosi    : out   std_logic;
		spi_miso    : in    std_logic;

		-- --------------------------------------------------------
		-- Ethernet PHY
		-- --------------------------------------------------------
		--
		-- I2C interface
		eth_mdc     : out   std_logic;
		eth_mdio    : inout std_logic;

		-- PHY interface
		eth_tx_clk  : in    std_logic;
		eth_rx_clk  : in    std_logic;
		eth_rstN    : out   std_logic;
		eth_col     : in    std_logic;
		eth_crs     : in    std_logic;
		eth_rx_er   : in    std_logic;
		eth_rx_dv   : in    std_logic;
		eth_tx_en   : out   std_logic;
		eth_txd     : out   std_logic_vector(3 downto 0);
		eth_rxd     : in    std_logic_vector(3 downto 0);

		-- --------------------------------------------------------
		-- MicroSD card slot
		-- --------------------------------------------------------
		--
		sd_clk      : out   std_logic;
		sd_cmd      : out   std_logic;
		sd_dat      : inout std_logic_vector(3 downto 0);

		-- --------------------------------------------------------
		-- Mobile DDR memory
		-- --------------------------------------------------------
		--
		-- Differential clock
		ddr_ck_p    : out   std_logic;
		ddr_ck_n    : out   std_logic;

		-- Controls
		ddr_csN     : out   std_logic;
		ddr_rasN    : out   std_logic;
		ddr_casN    : out   std_logic;
		ddr_weN     : out   std_logic;
		ddr_cke     : out   std_logic;

		-- Data mask (write byte-enable)
		ddr_dqm     : out   std_logic_vector( 1 downto 0);

		-- Data strobe
		ddr_dqs     : inout std_logic_vector( 1 downto 0);

		-- Address outputs
		ddr_a       : out   std_logic_vector(13 downto 0);
		ddr_ba      : out   std_logic_vector( 1 downto 0);

		-- Bidirectional 16-bit data bus
		ddr_dq      : inout std_logic_vector(15 downto 0);

		-- --------------------------------------------------------
		-- GPIO (expansion connector)
		-- --------------------------------------------------------
		--
		-- Reset from the expansion board
		exp_rstN    : in    std_logic;

		-- Expansion board present (when high)
		exp_present : in    std_logic;

		-- GPINs (with external pull-downs)
		exp_gpin    : in    std_logic_vector( 3 downto 0);

		-- GPIOs
		exp_gpio    : inout std_logic_vector(54 downto 4)
	);
end entity;

-- ----------------------------------------------------------------

architecture basic of bemicro_sdk is

	-- ------------------------------------------------------------
	-- Component
	-- ------------------------------------------------------------
	--
	component blink_leds is
		generic (
			CLK_FREQUENCY : real;
			BLINK_PERIOD  : real;
			LED_WIDTH     : integer;
			ACTIVE_HIGH   : boolean;
			LIB_VENDOR    : string;
			LIB_TYPE      : string
		);
		port (
			rstN : in  std_logic;
			clk  : in  std_logic;
			led  : out std_logic_vector(LED_WIDTH-1 downto 0)
		);
	end component;

	-- ------------------------------------------------------------
	-- Signals
	-- ------------------------------------------------------------
	--
	-- Reset and clock (rename the external signals)
	alias rstN is cpu_rstN;
	alias clk  is clkin_50MHz;

begin

	-- ------------------------------------------------------------
	-- Blink LEDs
	-- ------------------------------------------------------------
	--
	u1: blink_leds
		generic map (
			CLK_FREQUENCY => CLK_FREQUENCY,
			BLINK_PERIOD  => BLINK_PERIOD,
			LED_WIDTH     => 8,
			ACTIVE_HIGH   => false,
			LIB_VENDOR    => LIB_VENDOR,
			LIB_TYPE      => LIB_TYPE
		)
		port map (
			rstN => rstN,
			clk  => clk,
			led  => led
		);

	-- ------------------------------------------------------------
	-- Unused outputs
	-- ------------------------------------------------------------
	--
	-- SPI temperature sensor
	spi_sck  <= '0';
	spi_csN  <= '0';
	spi_mosi <= 'Z';

	-- Ethernet PHY
	eth_mdc   <= '0';
	eth_mdio  <= 'Z';
	eth_rstN  <= '0';
	eth_tx_en <= '0';
	eth_txd   <= (others => '0');

	-- MicroSD card slot
	sd_clk <= '0';
	sd_cmd <= '1';
	sd_dat <= (others => '1');

	-- Mobile DDR memory
	ddr_ck_p <= '0';
	ddr_ck_n <= '1';
	ddr_csN  <= '1';
	ddr_rasN <= '1';
	ddr_casN <= '1';
	ddr_weN  <= '1';
	ddr_cke  <= '0';
	ddr_dqm  <= (others => '1');
	ddr_dqs  <= (others => 'Z');
	ddr_a    <= (others => '0');
	ddr_ba   <= (others => '0');
	ddr_dq   <= (others => '0');

	-- GPIOs
	exp_gpio <= (others => 'Z');

end architecture;
