-- ----------------------------------------------------------------
-- lib_counter_altera.vhd
--
-- 9/28/2010 D. W. Hawkins (dwh@ovro.caltech.edu)
--
-- Altera implementation of the counter component.
--
-- ----------------------------------------------------------------
-- Notes:
-- ------
--
-- 1. Synthesis test results (9/28/2010)
--
--    Quartus 9.1
--
-- a) LIB_VENDOR    "ALTERA"
--    LIB_TYPE      "LPM"
--    LIB_FAMILY    "DEFAULT"
--    LIB_WIDTH      8
--    LIB_DIRECTION "UP" or "DOWN"
--
--    Target         Cyclone III EP3C16F256C8 (BeMicro device)
--
--    19 LEs total = 8 registers + 11 combinatorial
--
--    No timing constraints were applied.
--
-- b) Then change to:
--
--    LIB_VENDOR "GENERIC", or
--    LIB_VENDOR "ALTERA" and LIB_TYPE = "GENERIC"
--    LIB_DIRECTION "UP"
--
--    11 LEs total = 8 registers + 3 combinatorial
--
--    LIB_DIRECTION "DOWN"
--
--    12 LEs total = 8 registers + 4 combinatorial
--
--    Interestingly, the inferred logic uses fewer resources.
--    The opposite is observed in the Lattice synthesis tests
--    (see the notes in lib_counter_lattice.vhd)
--
-- ----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

-- Altera LPM components
library lpm;
use lpm.lpm_components.all;

-- Generic components
library lib_generic;

-- ----------------------------------------------------------------

entity lib_counter_altera is
	generic (
		-- Implementation selection
		LIB_TYPE      : string;
		LIB_FAMILY    : string;

		-- Component parameters
		LIB_WIDTH     : integer;
		LIB_DIRECTION : string
	);
	port (
		rst   : in  std_logic;
		clk   : in  std_logic;
		clr   : in  std_logic;
		load  : in  std_logic;
		en    : in  std_logic;
		d     : in  std_logic_vector(LIB_WIDTH-1 downto 0);
		q     : out std_logic_vector(LIB_WIDTH-1 downto 0);
		cout  : out std_logic
	);
end entity;

-- ----------------------------------------------------------------

architecture structure of lib_counter_altera is

	-- ------------------------------------------------------------
	-- Generic counter
	-- ------------------------------------------------------------
	--
	component lib_counter_generic is
		generic (
			-- Component parameters
			LIB_WIDTH     : integer;
			LIB_DIRECTION : string
		);
		port (
			rst   : in  std_logic;
			clk   : in  std_logic;
			clr   : in  std_logic;
			load  : in  std_logic;
			en    : in  std_logic;
			d     : in  std_logic_vector(LIB_WIDTH-1 downto 0);
			q     : out std_logic_vector(LIB_WIDTH-1 downto 0);
			cout  : out std_logic
		);
	end component;

begin

	-- ------------------------------------------------------------
	-- Generic VHDL counter
	-- ------------------------------------------------------------
	--
	g1: if (LIB_TYPE = "GENERIC") generate
		u1: lib_counter_generic
			generic map (
				LIB_WIDTH     => LIB_WIDTH,
				LIB_DIRECTION => LIB_DIRECTION
			)
			port map (
				rst   => rst,
				clk   => clk,
				clr   => clr,
				load  => load,
				en    => en,
				d     => d,
				q     => q,
				cout  => cout
			);
	end generate;

	-- ------------------------------------------------------------
	-- Default/LPM counter
	-- ------------------------------------------------------------
	--
	g2: if ((LIB_TYPE = "DEFAULT") or
	        (LIB_TYPE = "LPM"))    generate

		u1: lpm_counter
			generic map (
				LPM_WIDTH     => LIB_WIDTH,
				LPM_DIRECTION => LIB_DIRECTION
			)
			port map (
				aclr   => rst,
				clock  => clk,
				sclr   => clr,
				sload  => load,
				cnt_en => en,
				data   => d,
				q      => q,
				cout   => cout
			);
	end generate;

end architecture;