-- ----------------------------------------------------------------
-- lib_generic_pkg.vhd
--
-- 9/28/2010 D. W. Hawkins (dwh@ovro.caltech.edu)
--
-- Gernic (vendor independent) components interface.
--
-- This package implements vendor-independent interfaces to
-- components. Generics on the components are used to select
-- a generic or vendor-dependent implementation.
--
-- The use of these components in designs makes those designs
-- vendor neutral, while ensuring an efficient implementation
-- by using vendor-specific implementations. The timing and
-- resource benefits of vendor-specific versus inferred logic
-- can be analyzed using the VENDOR and IMPLEMENTATION
-- generics.
--
-- ----------------------------------------------------------------
-- Notes:
-- ------
--
-- 1. Component implementation naming conventions
--
--    This package defines the 'public' interface to components.
--
--    The implementation of the component is controlled via the
--    generics;
--
--     * LIB_VENDOR - selects the target vendor
--     * LIB_TYPE   - selects a vendor implementation
--     * LIB_FAMILY - selects a vendor device
--
--    Each of the lib_generic_pkg top-level components is
--    implemented using generate statements based on the
--    LIB_VENDOR generic.
--
--    The LIB_FAMILY and LIB_TYPE generics are strings that are
--    passed down to the vendor-specific implementation. The
--    purpose of the strings is to select a vendor component
--    instance that is compatible with the target hardware.
--
--    These additional generics are necessary, as different devices
--    from the same vendor do not always support the same
--    interface signals, eg. older Altera FLEX10K devices do not
--    support byte-enables on their internal RAM, whereas newer
--    Cyclone and Stratix devices do. Altera has two libraries
--    of generic components with varying support across device
--    families; LPM and MegaFunction. The library wraps both
--    RAM implementations, providing a common interface, and
--    ignores the byte-enables when LIB_TYPE = "LPM".
--    The simulation libraries for these vendor library components
--    sometimes requires a device family to be selected, and
--    the LIB_FAMILY generic can be used to pass that information.
--    The Lattice PMI library also has a family generic.
--
--    The LIB_VENDOR = "GENERIC" option allows the vendor tool
--    to infer the component. The LIB_TYPE generic is ignored
--    in this case. The LIB_VENDOR /= "GENERIC" and
--    LIB_TYPE = "GENERIC" option allows vendor-specific
--    refinement for inference from a generic VHDL implementation
--    eg. vendor-specific attributes or coding style.
--
--    The vendor-specific component naming convention is;
--
--    lib_<component>_<vendor>
--
--    eg. the lib_counter targeting an Altera device is implemented
--    by the component lib_counter_altera.
--
--    This naming convention is used, rather than architecture
--    names, eg. lib_counter_altera vs lib_counter(altera), so
--    that there is only one lib_counter architecture defined,
--    eg. lib_counter(top). This allows VHDL's default binding
--    rule (use the last architecture built) to select the
--    implementation for the lib_counter. This also allows a
--    design-specific implementation of a lib_counter to be
--    implemented and processed after the library to
--    override the component.
--
--    The lower-level vendor components are instantiated using
--    entity instantiations. If the vendor designs become too
--    complex to be written inline in a single component with
--    multiple generate statements, additional architectures or
--    named components (lib_<component>_<vendor>_<type>) can be
--    used. None of these 'private' components are expected to be
--    directly instantiated by users of the library.
--
-- 2. Generics
--
--    LIB_VENDOR:
--       "GENERIC" - generic VHDL implementation
--       "ALTERA"  - Altera implementation
--       "LATTICE" - Lattice implementation
--       "XILINX"  - Xilinx implementation
--
--    The implementation can be further refined once the vendor
--    has been selected, eg.,
--
--    LIB_VENDOR = "ALTERA"
--      LIB_TYPE:
--       "DEFAULT" - default implementation
--       "LPM"     - Library of Parameterized Modules (LPM)
--       "MF"      - MegaFunction
--
--      LIB_FAMILY:
--       "FLEX"    - FLEX10K devices
--       "CYCLONE" - Cyclone devices
--       "STRATIC" - Cyclone devices
--
--    LIB_VENDOR = "LATTICE"
--      LIB_TYPE:
--       "DEFAULT" - default implementation
--       "PMI"     - Parameterized Module Instantiation
--
--      LIB_FAMILY:
--       "XO"      - MachXO devices
--       "XP2"     - XP2 devices
--
--    The strings supported by LIB_FAMILY and LIB_TYPE depends on
--    the vendor libraries used to implement the component.
--
-- ----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

-- ----------------------------------------------------------------

package lib_generic_pkg is

	-- ------------------------------------------------------------
	-- Counter
	-- ------------------------------------------------------------
	--
	component lib_counter is
		generic (
			-- Implementation selection
			LIB_VENDOR    : string  := "GENERIC";
			LIB_TYPE      : string  := "DEFAULT";
			LIB_FAMILY    : string  := "DEFAULT";

			-- Component parameters
			LIB_WIDTH     : integer := 8;
			LIB_DIRECTION : string  := "UP"  -- "UP" or "DOWN"
		);
		port (
			rst   : in  std_logic;
			clk   : in  std_logic;
			clr   : in  std_logic := '0';
			load  : in  std_logic := '0';
			en    : in  std_logic := '1'; -- enable/carry-in
			d     : in  std_logic_vector(LIB_WIDTH-1 downto 0) := (others => '0');
			q     : out std_logic_vector(LIB_WIDTH-1 downto 0);
			cout  : out std_logic
		);
	end component;

end package;
