-- ----------------------------------------------------------------
-- blink_leds.vhd
--
-- 1/23/2011 D. W. Hawkins (dwh@ovro.caltech.edu)
--
-- A component to blink LEDs.
--
-- ----------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use ieee.math_real.all;

-- ----------------------------------------------------------------

entity blink_leds is
	generic (
		-- Clock frequency
		CLK_FREQUENCY : real := 50.0e6;

		-- LED LSB blink period
		BLINK_PERIOD  : real := 0.5;

		-- Number of LEDs
		LED_WIDTH     : integer := 8;

		-- LEDs are active high or low?
		ACTIVE_HIGH   : boolean := true
	);
	port (
		rstN : in  std_logic;
		clk  : in  std_logic;
		led  : out std_logic_vector(LED_WIDTH-1 downto 0)
	);
end entity;

-- ----------------------------------------------------------------

architecture mixed of blink_leds is

	-- ------------------------------------------------------------
	-- Components
	-- ------------------------------------------------------------
	--
	component up_counter is
		generic (
			WIDTH     : integer
		);
		port (
			rst   : in  std_logic;
			clk   : in  std_logic;
			q     : out std_logic_vector(WIDTH-1 downto 0)
		);
	end component;

	-- ------------------------------------------------------------
	-- Parameters
	-- ------------------------------------------------------------
	--
	-- Counter count value
	constant COUNT_VALUE : real := BLINK_PERIOD*CLK_FREQUENCY;

	-- Counter width required
	constant COUNT_WIDTH : integer :=
		integer(ceil(log2(COUNT_VALUE-1.0))) + LED_WIDTH - 1;

	-- ------------------------------------------------------------
	-- Signals
	-- ------------------------------------------------------------
	--
	-- Active high reset
	signal rst : std_logic;

	-- Counter
	signal count : std_logic_vector(COUNT_WIDTH-1 downto 0);

	-- Most significant bits of the counter
	alias led_count is
		count(COUNT_WIDTH-1 downto COUNT_WIDTH-LED_WIDTH);

begin

	-- ------------------------------------------------------------
	-- Generics
	-- ------------------------------------------------------------
	--
	-- Report the values of the generics.
	--
	assert false
		report
		       lf &
		       " => CLK_FREQUENCY = " & real'image(CLK_FREQUENCY) &
		       lf &
		       " => BLINK_PERIOD = " & real'image(BLINK_PERIOD) &
		       lf &
		       " => COUNT_VALUE = " & real'image(COUNT_VALUE) &
		       lf &
		       " => COUNT_WIDTH = " & integer'image(COUNT_WIDTH) &
		       lf
		severity note;

	-- ------------------------------------------------------------
	-- LED counter
	-- ------------------------------------------------------------
	--
	u1: up_counter
		generic map (
			WIDTH     => COUNT_WIDTH
		)
		port map (
			rst   => rst,
			clk   => clk,
			q     => count
		);

	rst <= not rstN;

	-- ------------------------------------------------------------
	-- LED output
	-- ------------------------------------------------------------
	--
	g1: if (ACTIVE_HIGH) generate
		led <= led_count;
	end generate;

	g2: if (not ACTIVE_HIGH) generate
		led <= not led_count;
	end generate;

end architecture;
